import { __decorate } from "tslib";
import { attr, nullableNumberConverter, observable } from "@microsoft/fast-element";
import { FormAssociated } from "../form-associated/index";
import { StartEnd } from "../patterns/start-end";
import { applyMixins } from "../utilities/index";
export var TextFieldAppearance;
(function (TextFieldAppearance) {
    TextFieldAppearance["filled"] = "filled";
    TextFieldAppearance["outline"] = "outline";
})(TextFieldAppearance || (TextFieldAppearance = {}));
export var TextFieldType;
(function (TextFieldType) {
    TextFieldType["email"] = "email";
    TextFieldType["password"] = "password";
    TextFieldType["tel"] = "tel";
    TextFieldType["text"] = "text";
    TextFieldType["url"] = "url";
})(TextFieldType || (TextFieldType = {}));
export class TextField extends FormAssociated {
    constructor() {
        super();
        this.appearance = TextFieldAppearance.outline;
        this.type = TextFieldType.text;
        this.proxy = document.createElement("input");
        this.proxy.setAttribute("type", this.type);
    }
    readOnlyChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.readOnly = this.readOnly;
        }
    }
    autofocusChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.autofocus = this.autofocus;
        }
    }
    placeholderChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.placeholder = this.placeholder;
        }
    }
    typeChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.type = this.type;
        }
    }
    listChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.setAttribute("list", this.list);
        }
    }
    maxlengthChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.maxLength = this.maxlength;
        }
    }
    minlengthChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.minLength = this.minlength;
        }
    }
    patternChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.pattern = this.pattern;
        }
    }
    sizeChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.size = this.size;
        }
    }
    spellcheckChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.spellcheck = this.spellcheck;
        }
    }
    valueChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.value = this.value;
        }
        this.$emit("change", this.value);
    }
    connectedCallback() {
        super.connectedCallback();
        if (this.autofocus) {
            this.focus();
        }
        this.setFormValue(this.value, this.value);
    }
    handleTextInput() {
        if (this.control && this.control.value) {
            this.value = this.control.value;
        }
    }
}
__decorate([
    attr
], TextField.prototype, "appearance", void 0);
__decorate([
    attr({ attribute: "readonly", mode: "boolean" })
], TextField.prototype, "readOnly", void 0);
__decorate([
    attr({ mode: "boolean" })
], TextField.prototype, "autofocus", void 0);
__decorate([
    attr
], TextField.prototype, "placeholder", void 0);
__decorate([
    attr
], TextField.prototype, "type", void 0);
__decorate([
    attr
], TextField.prototype, "list", void 0);
__decorate([
    attr({ converter: nullableNumberConverter })
], TextField.prototype, "maxlength", void 0);
__decorate([
    attr({ converter: nullableNumberConverter })
], TextField.prototype, "minlength", void 0);
__decorate([
    attr
], TextField.prototype, "pattern", void 0);
__decorate([
    attr({ converter: nullableNumberConverter })
], TextField.prototype, "size", void 0);
__decorate([
    attr({ mode: "boolean" })
], TextField.prototype, "spellcheck", void 0);
__decorate([
    observable
], TextField.prototype, "defaultSlottedNodes", void 0);
applyMixins(TextField, StartEnd);
