import { Behavior } from '@microsoft/fast-element';
import { DecoratorAttributeConfiguration } from '@microsoft/fast-element';
import { Direction } from '@microsoft/fast-web-utilities';
import { ElementStyles } from '@microsoft/fast-element';
import { FASTElement } from '@microsoft/fast-element';
import { Orientation } from '@microsoft/fast-web-utilities';
import { PartialFASTElementDefinition } from '@microsoft/fast-element';

export declare class Accordion extends FASTElement {
    expandmode: AccordionExpandMode;
    accordionItems: HTMLElement[];
    accordionItemsChanged(oldValue: any, newValue: any): void;
    private activeid;
    private activeItemIndex;
    private accordionIds;
    private change;
    private setItems;
    private resetItems;
    private removeItemListeners;
    private activeItemChange;
    private getItemIds;
    private isSingleExpandMode;
    private handleItemKeyDown;
    private adjust;
    private focusItem;
}

export declare enum AccordionExpandMode {
    single = "single",
    multi = "multi"
}

export declare class AccordionItem extends FASTElement {
    headinglevel: 1 | 2 | 3 | 4 | 5 | 6;
    expanded: boolean;
    id: string;
    expandbutton: HTMLElement;
    clickHandler: (e: MouseEvent) => void;
    private change;
}

export declare interface AccordionItem extends StartEnd {
}

export declare const AccordionItemTemplate: import("@microsoft/fast-element").ViewTemplate<AccordionItem, any>;

export declare const AccordionTemplate: import("@microsoft/fast-element").ViewTemplate<Accordion, any>;

export declare class Anchor extends FASTElement {
    appearance: AnchorAppearance;
    download: string;
    href: string;
    hreflang: string;
    ping: string;
    referrerpolicy: string;
    rel: string;
    target: "_self" | "_blank" | "_parent" | "_top";
    type: string;
}

export declare interface Anchor extends StartEnd {
}

export declare type AnchorAppearance = ButtonAppearance | "hypertext";

export declare const AnchorTemplate: import("@microsoft/fast-element").ViewTemplate<Anchor, any>;

/**
 * Apply mixins to a constructor.
 *
 * https://www.typescriptlang.org/docs/handbook/mixins.html
 */
export declare function applyMixins(derivedCtor: any, ...baseCtors: any[]): void;

export declare class Badge extends FASTElement {
    fill: string;
    color: string;
    circular: boolean;
}

export declare const BadgeTemplate: import("@microsoft/fast-element").ViewTemplate<Badge, any>;

export declare class BaseProgress extends FASTElement {
    value: number;
    min: number;
    max: number;
    paused: any;
}

export declare class Button extends FormAssociated<HTMLInputElement> {
    appearance: ButtonAppearance;
    autofocus: boolean;
    formId: string;
    formaction: string;
    private formactionChanged;
    formenctype: string;
    private formenctypeChanged;
    formmethod: string;
    private formmethodChanged;
    formnovalidate: boolean;
    private formnovalidateChanged;
    formtarget: "_self" | "_blank" | "_parent" | "_top";
    private formtargetChanged;
    name: string;
    type: "submit" | "reset" | "button";
    private typeChanged;
    value: string;
    private valueChanged;
    protected proxy: HTMLInputElement;
    constructor();
    connectedCallback(): void;
}

export declare interface Button extends StartEnd {
}

export declare type ButtonAppearance = "accent" | "lightweight" | "neutral" | "outline" | "stealth";

export declare const ButtonTemplate: import("@microsoft/fast-element").ViewTemplate<Button, any>;

export declare class Card extends FASTElement {
}

export declare const CardTemplate: import("@microsoft/fast-element").ViewTemplate<Card, any>;

export declare class Checkbox extends FormAssociated<HTMLInputElement> {
    readOnly: boolean;
    private readOnlyChanged;
    /**
     * The element's value to be included in form submission when checked.
     * Default to "on" to reach parity with input[type="checkbox"]
     */
    value: string;
    private valueChanged;
    /**
     * Provides the default checkedness of the input element
     * Passed down to proxy
     */
    checkedAttribute: boolean;
    private checkedAttributeChanged;
    defaultSlottedNodes: Node[];
    /**
     * Initialized to the value of the checked attribute. Can be changed independently of the "checked" attribute,
     * but changing the "checked" attribute always additionally sets this value.
     */
    defaultChecked: boolean;
    private defaultCheckedChanged;
    /**
     * The checked state of the control
     */
    checked: boolean;
    private checkedChanged;
    protected proxy: HTMLInputElement;
    /**
     * The indeterminate state of the control
     */
    indeterminate: boolean;
    /**
     * Tracks whether the "checked" property has been changed.
     * This is necessary to provide consistent behavior with
     * normal input checkboxes
     */
    private dirtyChecked;
    /**
     * Set to true when the component has constructed
     */
    private constructed;
    constructor();
    connectedCallback(): void;
    private updateForm;
    keypressHandler: (e: KeyboardEvent) => void;
    clickHandler: (e: MouseEvent) => void;
}

export declare const CheckboxTemplate: import("@microsoft/fast-element").ViewTemplate<Checkbox, any>;

/**
 * Retrieves the "parent" element of a node, ignoring DOM tree boundries.
 * When the "parent" of a node is a shadow-root, it will return the host
 * element of the shadow root.
 * @param element The element to retrieve the parent element of
 */
export declare function composedParent<T extends HTMLElement>(element: T): HTMLElement | null;

export declare type CSSCustomPropertyBehavior = Behavior & CSSCustomPropertyDefinition;

/**
 * Create a CSS Custom Property behavior.
 * @param name The name of the CSS custom property
 * @param value The value or value resolver of the custom property
 * @param host A function to resolve the element to host the CSS custom property
 */
export declare function cssCustomPropertyBehaviorFactory(name: string, value: string | ((...arg: any[]) => string), host: (source: typeof FASTElement & HTMLElement) => Partial<CSSCustomPropertyTarget> | null): CSSCustomPropertyBehavior;

export declare interface CSSCustomPropertyDefinition {
    /**
     * The custom property name
     */
    name: string;
    /**
     * The value of the custom property or a function that resolves the value
     */
    value: string | ((...args: any[]) => string);
}

export declare interface CSSCustomPropertyTarget {
    registerCSSCustomProperty(behavior: CSSCustomPropertyDefinition): void;
    unregisterCSSCustomProperty(behavior: CSSCustomPropertyDefinition): void;
    disconnectedCSSCustomPropertyRegistry: CSSCustomPropertyDefinition[] | void;
}

export declare type CSSDisplayPropertyValue = "block" | "contents" | "flex" | "grid" | "inherit" | "initial" | "inline" | "inline-block" | "inline-flex" | "inline-grid" | "inline-table" | "list-item" | "none" | "run-in" | "table" | "table-caption" | "table-cell" | "table-column" | "table-column-group" | "table-footer-group" | "table-header-group" | "table-row" | "table-row-group";

export declare interface DecoratorDesignSystemPropertyConfiguration extends Omit<DecoratorAttributeConfiguration, "attribute"> {
    attribute?: string | false;
    cssCustomProperty?: string | false;
    default: any;
}

export declare interface DesignSystemConsumer {
    provider: DesignSystemProvider | null;
}

/**
 * Behavior to connect an element to the nearest design-system provider
 */
export declare const designSystemConsumerBehavior: Behavior;

/**
 * Decorator to declare a property as a design-system property.
 * Accepts a config object with the following:
 *
 * default:
 * The default value of the property. Will be assigned when the use-defaults attribute is used.
 *
 * attribute?:
 * The HTML attribute to map the property to - defaults to the property name.
 *
 * mode?:
 * The attr mode - see https://github.com/microsoft/fast-dna/blob/master/packages/web-components/fast-element/docs/guide/building-components.md#customizing-attributes
 * Defaults to "fromView"
 *
 * converter?:
 * The attr converter - see https://github.com/microsoft/fast-dna/blob/master/packages/web-components/fast-element/docs/guide/building-components.md#customizing-attributes
 *
 * cssCustomProperty?:
 * An optional property to control the name of the css custom property being created.
 * If omitted, the css custom property will share a name with attribute if specified, otherwise the property name being decorated.
 * If assigned a false value, no css custom property will be created.
 */
export declare function designSystemProperty<T extends DesignSystemProvider>(config: DecoratorDesignSystemPropertyConfiguration): (source: T, property: string) => void;

export declare class DesignSystemProvider extends FASTElement implements CSSCustomPropertyTarget, DesignSystemConsumer {
    /**
     * Stores a list of all element tag-names that associated
     * to design-system-providers
     */
    private static _tagNames;
    /**
     * Read all tag-names that are associated to
     * design-system-providers
     */
    static get tagNames(): string[];
    /**
     * Determines if an element is a DesignSystemProvider
     * @param el The element to test
     */
    static isDesignSystemProvider(el: HTMLElement | DesignSystemProvider): el is DesignSystemProvider;
    /**
     * Finds the closest design-system-provider
     * to an element.
     */
    static findProvider(el: HTMLElement & Partial<DesignSystemConsumer>): DesignSystemProvider | null;
    /**
     * Registers a tag-name to be associated with
     * the design-system-provider class
     */
    static registerTagName(tagName: string): void;
    /**
     * Allows other components to identify this as a provider.
     * Using instanceof DesignSystemProvider did not seem to work.
     */
    readonly isDesignSystemProvider = true;
    /**
     * The design-system object.
     * This is "observable" but will notify on object mutation
     * instead of object assignment
     */
    designSystem: {};
    /**
     * Applies the default design-system values to the instance where properties
     * are not explicitly assigned. This is generally used to set the root design
     * system context.
     */
    useDefaults: boolean;
    private useDefaultsChanged;
    provider: DesignSystemProvider | null;
    private providerChanged;
    /**
     * Stores all CSSCustomPropertyDefinitions registered with the provider.
     */
    private cssCustomPropertyDefinitions;
    /**
     * Track all design system property names so we can react to changes
     * in those properties. Do not initialize or it will clobber value stored
     * by the decorator.
     */
    designSystemProperties: {
        [propertyName: string]: Required<Pick<DecoratorDesignSystemPropertyConfiguration, "cssCustomProperty" | "default">>;
    };
    /**
     * Allows CSSCustomPropertyDefinitions to register on this element *before* the constructor
     * has run and the registration APIs exist. This can manifest when the DOM
     * is parsed (and custom element tags exist in the DOM) before the script defining the custom elements
     * and elements is parsed, and the elements using the CSSCustomPropertyBehaviors
     * are defined before this DesignSystemProvider.
     */
    disconnectedCSSCustomPropertyRegistry: CSSCustomPropertyDefinition[];
    /**
     * The target of CSSCustomPropertyDefinitions registered
     * with the provider. This will be #1 when adoptedStyleSheets are supported
     * and #2 when they are not.
     *
     * 1. The `style` property of a CSSStyleRule on an adoptedStyleSheet
     * 2. The `style` property of the element, resulting in inline styles
     */
    private customPropertyTarget;
    /**
     * Handle changes to design-system-provider IDL and content attributes
     * that reflect to design-system properties.
     */
    private attributeChangeHandler;
    /**
     * Handle changes to the local design-system property.
     */
    private localDesignSystemChangeHandler;
    /**
     * Handle changes to the upstream design-system provider
     */
    private providerDesignSystemChangeHandler;
    constructor();
    connectedCallback(): void;
    /**
     * Register a CSSCustomPropertyDefinition with the design system provider.
     * Registering a CSSCustomPropertyDefinition will create the CSS custom property.
     */
    registerCSSCustomProperty(behavior: CSSCustomPropertyDefinition): void;
    /**
     * Unregister a CSSCustomPropertyDefinition. If all registrations of the definition
     * are unregistered, the CSS custom property will be removed.
     */
    unregisterCSSCustomProperty(behavior: CSSCustomPropertyDefinition): void;
    /**
     * Writes all CSS custom property definitions to the design system provider.
     */
    private writeCustomProperties;
    /**
     * Writes a CSS custom property to the design system provider,
     * evaluating any function values with the design system.
     */
    private setCustomProperty;
    /**
     * Removes a CSS custom property from the provider.
     */
    private deleteCustomProperty;
    /**
     * Evaluates a CSSCustomPropertyDefinition with the current design system.
     */
    evaluate(definition: CSSCustomPropertyDefinition): string;
    /**
     * Synchronize the provider's design system with the local
     * overrides. Any value defined on the instance will take priority
     * over the value defined by the provider
     */
    private syncDesignSystemWithProvider;
    private isValidDesignSystemValue;
}

/**
 * Defines a design-system-provider custom element
 */
export declare function designSystemProvider(nameOrDef: string | PartialFASTElementDefinition): <T extends typeof DesignSystemProvider>(providerCtor: T) => void;

export declare const DesignSystemProviderTemplate: import("@microsoft/fast-element").ViewTemplate<DesignSystemProvider, any>;

export declare class Dialog extends FASTElement {
    modal: boolean;
    hidden: boolean;
    trapFocus: boolean;
    ariaDescribedby: string;
    ariaLabelledby: string;
    ariaLabel: string;
    dialog: HTMLDivElement;
    private tabbableElements;
    private observer;
    dismiss(): void;
    connectedCallback(): void;
    disconnectedCallback(): void;
    private onChildListChange;
    private trapFocusChanged;
    private handleDocumentKeydown;
    private handleDocumentFocus;
    private handleTabKeyDown;
    /**
     * focus on first element of tab queue
     */
    private focusFirstElement;
    /**
     * we should only focus if focus has not already been brought to the dialog
     */
    private shouldForceFocus;
    /**
     * TODO: Issue #2742 - https://github.com/microsoft/fast-dna/issues/2742
     * This is a placeholder function to check if the hidden attribute is present
     * Currently there is not support for boolean attributes.
     * Once support is added, we will simply use this.hidden.
     */
    private isDialogHidden;
    /**
     * TODO: Issue #2742 - https://github.com/microsoft/fast-dna/issues/2742
     * This is a placeholder function to check if the trapFocus attribute is present
     * Currently there is not support for boolean attributes.
     * Once support is added, we will simply use this.trapFocus.
     */
    private shouldDialogTrapFocus;
}

export declare const DialogTemplate: import("@microsoft/fast-element").ViewTemplate<Dialog, any>;

export declare const disabledCursor = "not-allowed";

/**
 * Apply the common styles for every component
 * @param display The CSS display property value
 */
export declare function display(displayValue: CSSDisplayPropertyValue): string;

export declare class Divider extends FASTElement {
    role: DividerRole;
}

export declare enum DividerRole {
    separator = "separator",
    presentation = "presentation"
}

export declare const DividerTemplate: import("@microsoft/fast-element").ViewTemplate<Divider, any>;

/**
 * Source:
 * https://html.spec.whatwg.org/multipage/custom-elements.html#elementinternals
 */
declare interface ElementInternals {
    /**
     * Returns the form owner of internals target element.
     */
    readonly form: HTMLFormElement | null;
    /**
     * Returns a NodeList of all the label elements that internals target element is associated with.
     */
    readonly labels: NodeList;
    /**
     * Returns the error message that would be shown to the user if internals target element was to be checked for validity.
     */
    readonly validationMessage: string;
    /**
     * Returns the ValidityState object for internals target element.
     */
    readonly validity: ValidityState;
    /**
     * Returns true if internals target element will be validated when the form is submitted; false otherwise.
     */
    readonly willValidate: boolean;
    /**
     * Returns true if internals target element has no validity problems; false otherwise. Fires an invalid event at the element in the latter case.
     */
    checkValidity(): boolean;
    /**
     * Returns true if internals target element has no validity problems; otherwise,
     * returns false, fires an invalid event at the element, and (if the event isn't canceled) reports the problem to the user.
     */
    reportValidity(): boolean;
    /**
     * Sets both the state and submission value of internals target element to value.
     *
     * While "null" isn't enumerated as a argument type (here)[https://html.spec.whatwg.org/multipage/custom-elements.html#the-elementinternals-interface],
     * In practice it appears to remove the value from the form data on submission. Adding it as a valid type here
     * becuase that capability is required for checkbox and radio types
     */
    setFormValue(value: File | string | FormData | null, state?: File | string | FormData | null): void;
    /**
     * Marks internals target element as suffering from the constraints indicated by the flags argument,
     * and sets the element's validation message to message.
     * If anchor is specified, the user agent might use
     * it to indicate problems with the constraints of internals target
     * element when the form owner is validated interactively or reportValidity() is called.
     */
    setValidity(flags: ValidityStateFlags, message?: string, anchor?: HTMLElement_2): void;
}

declare let ElementInternals: {
    prototype: ElementInternals;
    new (): ElementInternals;
};

export declare const endTemplate: import("@microsoft/fast-element").ViewTemplate<StartEnd, any>;

export declare class Flipper extends FASTElement {
    disabled: boolean;
    hiddenFromAT: boolean;
    direction: FlipperDirection;
}

export declare enum FlipperDirection {
    next = "next",
    previous = "previous"
}

export declare const FlipperTemplate: import("@microsoft/fast-element").ViewTemplate<Flipper, any>;

/**
 * The string representing the focus selector to be used. Value
 * will be "focus-visible" when https://drafts.csswg.org/selectors-4/#the-focus-visible-pseudo
 * is supported and "focus" when it is not.
 */
export declare const focusVisible: string;

/**
 * Applies ElementStyles to a FASTElement when the
 * forced-colors media query is matched, removes the
 * ElementStyles when the query is no-longer matched
 */
export declare const forcedColorsStylesheetBehavior: (sheet: ElementStyles) => Readonly<{
    query: MediaQueryList;
    cache: WeakMap<(new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }, ((this: MediaQueryList) => void)[] | ((this: MediaQueryList) => void)>;
    sheet: ElementStyles;
    constructListener(this: MatchMediaStyleSheetBehavior, source: (new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }, sheet: ElementStyles): MediaQueryListListener;
    bind(this: MatchMediaStyleSheetBehavior, source: (new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }): void;
    unbind(this: MatchMediaStyleSheetBehavior, source: (new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }): void;
}>;

/**
 * Disable member ordering to keep property callbacks
 * grouped with property declaration
 */
declare abstract class FormAssociated<T extends HTMLInputElement | HTMLTextAreaElement | HTMLSelectElement> extends FASTElement {
    /**
     * Must evaluate to true to enable elementInternals.
     * Feature detects API support and resolve respectively
     */
    static get formAssociated(): boolean;
    /**
     * Returns the validty state of the element
     */
    get validity(): ValidityState;
    /**
     * Retrieve a reference to the associated form.
     * Returns null if not associated to any form.
     */
    get form(): HTMLFormElement | null;
    /**
     * Retrieve the localized validation message,
     * or custom validation message if set.
     */
    get validationMessage(): string;
    /**
     * Whether the element will be validated when the
     * form is submitted
     */
    get willValidate(): boolean;
    /**
     * A reference to all associated label elements
     */
    get labels(): ReadonlyArray<Node>;
    /**
     * The value of the element to be associated with the form
     */
    value: string;
    disabled: boolean;
    protected disabledChanged(): void;
    name: string;
    protected nameChanged(): void;
    /**
     * Require the field prior to form submission
     */
    required: boolean;
    protected requiredChanged(): void;
    /**
     * The proxy element provided by
     */
    protected abstract proxy: T;
    /**
     * The element internals object. Will only exist
     * in browsers supporting the attachInternals API
     */
    protected elementInternals: ElementInternals;
    /**
     * These are events that are still fired by the proxy
     * element based on user / programmatic interaction.
     *
     * The proxy implementation should be transparent to
     * the app author, so block these events from emitting.
     */
    private proxyEventsToBlock;
    constructor();
    connectedCallback(): void;
    disconnectedCallback(): void;
    /**
     * Return the current validity of the element
     */
    checkValidity(): boolean;
    /**
     * Return the current validity of the element.
     * If false, fires an invalid event at the element.
     */
    reportValidity(): boolean;
    /**
     * Set the validity of the control. In cases when the elementInternals object is not
     * available (and the proxy element is used to report validity), this function will
     * do nothing unless a message is provided, at which point the setCustomValidity method
     * of the proxy element will be invoked with the provided message.
     * @param flags Validity flags
     * @param message Optional message to supply
     * @param anchor Optional element used by UA to display an interactive validation UI
     */
    setValidity(flags: ValidityStateFlags, message?: string, anchor?: HTMLElement_2): void;
    /**
     * Invoked when a connected component's form or fieldset has it's disabled
     * state changed.
     * @param disabled the disabled value of the form / fieldset
     */
    formDisabledCallback(disabled: boolean): void;
    /**
     *
     * @param value The value to set
     * @param state The state object provided to during session restores and when autofilling.
     */
    protected setFormValue(value: File | string | FormData | null, state?: File | string | FormData | null): void;
    protected keypressHandler(e: KeyboardEvent): void;
    /**
     * Used to stop propagation of proxy element events
     * @param e Event object
     */
    private stopPropagation;
}

export declare const hidden = "\n    :host([hidden]) {\n        display: none;\n    }\n";

declare interface HTMLElement_2 {
    attachInternals?(): ElementInternals;
}

/**
 * Determines if the element has a design-system-provider context
 * @param element
 */
export declare function isDesignSystemConsumer(element: HTMLElement | DesignSystemConsumer): element is DesignSystemConsumer;

declare interface MatchMediaStyleSheetBehavior extends Behavior {
    query: MediaQueryList;
    cache: WeakMap<typeof FASTElement, MediaQueryListListener | Array<MediaQueryListListener>>;
    sheet: ElementStyles;
    constructListener: (source: typeof FASTElement, sheet: ElementStyles) => MediaQueryListListener;
}

/**
 * Construct a behavior to conditionally apply a stylesheet based
 * on a MediaQueryList
 */
export declare function matchMediaStylesheetBehaviorFactory(query: MediaQueryList): (sheet: ElementStyles) => Readonly<{
    query: MediaQueryList;
    cache: WeakMap<(new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }, ((this: MediaQueryList) => void)[] | ((this: MediaQueryList) => void)>;
    sheet: ElementStyles;
    constructListener(this: MatchMediaStyleSheetBehavior, source: (new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }, sheet: ElementStyles): MediaQueryListListener;
    bind(this: MatchMediaStyleSheetBehavior, source: (new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }): void;
    unbind(this: MatchMediaStyleSheetBehavior, source: (new () => HTMLElement & FASTElement) & {
        from<TBase extends {
            new (): HTMLElement;
            prototype: HTMLElement;
        }>(BaseType: TBase): new () => InstanceType<TBase> & FASTElement;
        define<TType extends Function>(Type: TType, nameOrDef?: string | import("@microsoft/fast-element").PartialFASTElementDefinition | undefined): TType;
        getDefinition<T extends Function>(Type: T): import("@microsoft/fast-element").FASTElementDefinition | undefined;
    }): void;
}>;

declare type MediaQueryListListener = (this: MediaQueryList, ev?: MediaQueryListEvent) => any;

export declare class Menu extends FASTElement {
    items: HTMLSlotElement;
    private itemsChanged;
    private menuItems;
    /**
     * The index of the focusable element in the items array
     * defaults to -1
     */
    private focusIndex;
    private static focusableElementRoles;
    connectedCallback(): void;
    focus(): void;
    handleMenuKeyDown(e: KeyboardEvent): void | boolean;
    /**
     * if focus is moving out of the menu, reset to a stable initial state
     */
    handleFocusOut: (e: FocusEvent) => void;
    private setItems;
    private resetItems;
    /**
     * get an array of valid DOM children
     */
    private domChildren;
    /**
     * check if the item is a menu item
     */
    private isMenuItemElement;
    /**
     * check if the item is disabled
     */
    private isDisabledElement;
    /**
     * check if the item is focusable
     */
    private isFocusableElement;
    private handleMenuItemFocus;
    private setFocus;
}

export declare class MenuItem extends FASTElement {
    disabled: boolean;
    expanded: boolean;
    role: MenuItemRole;
    checked: boolean;
    handleMenuItemKeyDown: (e: KeyboardEvent) => boolean;
    handleMenuItemClick: (e: MouseEvent) => void;
    private change;
}

export declare interface MenuItem extends StartEnd {
}

export declare enum MenuItemRole {
    menuitem = "menuitem",
    menuitemcheckbox = "menuitemcheckbox",
    menuitemradio = "menuitemradio"
}

export declare const MenuItemTemplate: import("@microsoft/fast-element").ViewTemplate<MenuItem, any>;

export declare const MenuTemplate: import("@microsoft/fast-element").ViewTemplate<Menu, any>;

export declare const ProgressRingTemplate: import("@microsoft/fast-element").ViewTemplate<BaseProgress, any>;

export declare const ProgressTemplate: import("@microsoft/fast-element").ViewTemplate<BaseProgress, any>;

export declare class Radio extends FormAssociated<HTMLInputElement> implements RadioControl {
    readOnly: boolean;
    private readOnlyChanged;
    name: string;
    protected nameChanged(): void;
    /**
     * The element's value to be included in form submission when checked.
     * Default to "on" to reach parity with input[type="radio"]
     */
    value: string;
    private valueChanged;
    /**
     * Provides the default checkedness of the input element
     * Passed down to proxy
     */
    checkedAttribute: boolean;
    private checkedAttributeChanged;
    defaultSlottedNodes: Node[];
    /**
     * Initialized to the value of the checked attribute. Can be changed independently of the "checked" attribute,
     * but changing the "checked" attribute always additionally sets this value.
     */
    defaultChecked: boolean;
    private defaultCheckedChanged;
    /**
     * The checked state of the control
     */
    checked: boolean;
    private checkedChanged;
    protected proxy: HTMLInputElement;
    /**
     * Tracks whether the "checked" property has been changed.
     * This is necessary to provide consistent behavior with
     * normal input radios
     */
    private dirtyChecked;
    constructor();
    connectedCallback(): void;
    private updateForm;
    keypressHandler: (e: KeyboardEvent) => void;
    clickHandler: (e: MouseEvent) => void;
}

export declare interface RadioControl {
    checked: boolean;
    disabled: boolean;
    readOnly: boolean;
    focus: () => void;
    setAttribute: (name: string, value: string) => void;
    getAttribute: (name: string) => string | null;
}

export declare class RadioGroup extends FASTElement {
    readOnly: boolean;
    private readOnlyChanged;
    disabled: boolean;
    private disabledChanged;
    name: string;
    protected nameChanged(): void;
    value: string;
    orientation: Orientation;
    slottedRadioButtons: RadioControl[];
    private selectedRadio;
    private focusedRadio;
    private parentToolbar;
    private isInsideToolbar;
    constructor();
    connectedCallback(): void;
    private getFilteredRadioButtons;
    private keypressHandler;
    private radioChangeHandler;
    private moveToRadioByIndex;
    private moveRightOffGroup;
    private moveLeftOffGroup;
    private focusOutHandler;
    private clickHandler;
    private shouldMoveOffGroupToTheRight;
    private shouldMoveOffGroupToTheLeft;
    private checkFocusedRadio;
    keydownHandler: (e: KeyboardEvent) => void;
}

export declare const RadioGroupTemplate: import("@microsoft/fast-element").ViewTemplate<RadioGroup, any>;

export declare const RadioTemplate: import("@microsoft/fast-element").ViewTemplate<Radio, any>;

export declare class Slider extends FormAssociated<HTMLInputElement> implements SliderConfiguration {
    readOnly: boolean;
    private readOnlyChanged;
    track: HTMLDivElement;
    thumb: HTMLDivElement;
    direction: Direction;
    isDragging: boolean;
    position: string;
    trackWidth: number;
    trackMinWidth: number;
    trackHeight: number;
    trackMinHeight: number;
    /**
     * The element's value to be included in form submission changed.
     */
    value: string;
    private valueChanged;
    /**
     * Min allowed value default is 0
     */
    min: number;
    private minChanged;
    /**
     * Max allowed value default is 10
     */
    max: number;
    private maxChanged;
    /**
     * Value to increment or decrement via arrow keys, mouse click or drag
     */
    step: number;
    private stepChanged;
    /**
     * Orientation value, horizontal | vertical
     */
    orientation: Orientation;
    private orientationChanged;
    /**
     * mode value, default singleValue
     */
    mode: SliderMode;
    protected proxy: HTMLInputElement;
    constructor();
    connectedCallback(): void;
    disconnectedCallback(): void;
    increment: () => void;
    decrement: () => void;
    protected keypressHandler: (e: KeyboardEvent) => void;
    private setThumbPositionForOrientation;
    private getDirection;
    private setupTrackConstraints;
    private setupListeners;
    private setupDefaultValue;
    private updateForm;
    /**
     *  Handle mouse moves during a thumb drag operation
     */
    private handleThumbMouseDown;
    /**
     *  Handle mouse moves during a thumb drag operation
     */
    private handleMouseMove;
    private calculateNewValue;
    /**
     * Handle a window mouse up during a drag operation
     */
    private handleWindowMouseUp;
    private stopDragging;
    private clickHandler;
    private convertToConstrainedValue;
}

export declare interface SliderConfiguration {
    max: number;
    min: number;
    orientation?: Orientation;
    direction?: Direction;
    disabled?: boolean;
}

export declare class SliderLabel extends FASTElement {
    positionStyle: string;
    root: HTMLDivElement;
    position: string;
    private positionChanged;
    hideMark: boolean;
    disabled: boolean;
    sliderOrientation: Orientation;
    sliderMinPosition: number;
    sliderMaxPosition: number;
    sliderDirection: Direction;
    private notifier;
    connectedCallback(): void;
    disconnectedCallback(): void;
    handleChange(source: any, propertyName: string): void;
    private isSliderConfig;
    private getSliderConfiguration;
    private positionAsStyle;
}

export declare const SliderLabelTemplate: import("@microsoft/fast-element").ViewTemplate<SliderLabel, any>;

export declare enum SliderMode {
    singleValue = "single-value"
}

export declare const SliderTemplate: import("@microsoft/fast-element").ViewTemplate<Slider, any>;

export declare class StartEnd {
    start: HTMLSlotElement;
    startContainer: HTMLSpanElement;
    handleStartContentChange(): void;
    end: HTMLSlotElement;
    endContainer: HTMLSpanElement;
    handleEndContentChange(): void;
}

export declare const startTemplate: import("@microsoft/fast-element").ViewTemplate<StartEnd, any>;

export declare class Switch extends FormAssociated<HTMLInputElement> {
    readOnly: boolean;
    private readOnlyChanged;
    /**
     * The element's value to be included in form submission when checked.
     * Default to "on" to reach parity with input[type="checkbox"]
     */
    value: string;
    private valueChanged;
    checkedAttribute: boolean;
    private checkedAttributeChanged;
    defaultSlottedNodes: Node[];
    /**
     * Initialized to the value of the checked attribute. Can be changed independently of the "checked" attribute,
     * but changing the "checked" attribute always additionally sets this value.
     */
    defaultChecked: boolean;
    private defaultCheckedChanged;
    /**
     * The checked state of the control
     */
    checked: boolean;
    private checkedChanged;
    protected proxy: HTMLInputElement;
    /**
     * Tracks whether the "checked" property has been changed.
     * This is necessary to provide consistent behavior with
     * normal input checkboxes
     */
    private dirtyChecked;
    constructor();
    connectedCallback(): void;
    private updateForm;
    keypressHandler: (e: KeyboardEvent) => void;
    clickHandler: (e: MouseEvent) => void;
}

export declare const SwitchTemplate: import("@microsoft/fast-element").ViewTemplate<Switch, any>;

export declare class Tab extends FASTElement {
}

export declare class TabPanel extends FASTElement {
}

export declare const TabPanelTemplate: import("@microsoft/fast-element").ViewTemplate<TabPanel, any>;

export declare class Tabs extends FASTElement {
    orientation: TabsOrientation;
    activeid: string;
    tabs: HTMLElement[];
    tabsChanged(): void;
    tabpanels: HTMLElement[];
    tabpanelsChanged(): void;
    activeindicator: boolean;
    activeIndicatorRef: HTMLElement;
    activetab: HTMLElement;
    private prevActiveTabIndex;
    private activeTabIndex;
    private ticking;
    private tabIds;
    private tabpanelIds;
    private change;
    private getActiveIndex;
    private setTabs;
    private setTabPanels;
    private getTabIds;
    private getTabPanelIds;
    private setComponent;
    private handleTabClick;
    private isHorizontal;
    private handleTabKeyDown;
    private handleActiveIndicatorPosition;
    private animateActiveIndicator;
    private adjust;
    private focusTab;
    constructor();
}

export declare interface Tabs extends StartEnd {
}

export declare enum TabsOrientation {
    vertical = "vertical",
    horizontal = "horizontal"
}

export declare const TabsTemplate: import("@microsoft/fast-element").ViewTemplate<Tabs, any>;

export declare const TabTemplate: import("@microsoft/fast-element").ViewTemplate<Tab, any>;

export declare class TextArea extends FormAssociated<HTMLTextAreaElement> {
    appearance: TextAreaAppearance;
    readOnly: boolean;
    private readOnlyChanged;
    resize: TextAreaResize;
    textarea: HTMLTextAreaElement;
    autofocus: boolean;
    private autofocusChanged;
    cols: number;
    formId: string;
    list: string;
    private listChanged;
    maxlength: number;
    private maxlengthChanged;
    minlength: number;
    private minlengthChanged;
    name: string;
    placeholder: string;
    rows: number;
    spellcheck: boolean;
    private spellcheckChanged;
    defaultSlottedNodes: Node[];
    valueChanged(): void;
    protected proxy: HTMLTextAreaElement;
    connectedCallback(): void;
    handleTextInput: () => void;
}

export declare enum TextAreaAppearance {
    filled = "filled",
    outline = "outline"
}

export declare enum TextAreaResize {
    none = "none",
    both = "both",
    horizontal = "horizontal",
    vertical = "vertical"
}

export declare const TextAreaTemplate: import("@microsoft/fast-element").ViewTemplate<TextArea, any>;

export declare class TextField extends FormAssociated<HTMLInputElement> {
    appearance: TextFieldAppearance;
    readOnly: boolean;
    private readOnlyChanged;
    autofocus: boolean;
    private autofocusChanged;
    placeholder: string;
    private placeholderChanged;
    type: TextFieldType;
    private typeChanged;
    list: string;
    private listChanged;
    maxlength: number;
    private maxlengthChanged;
    minlength: number;
    private minlengthChanged;
    pattern: string;
    private patternChanged;
    size: number;
    private sizeChanged;
    spellcheck: boolean;
    private spellcheckChanged;
    defaultSlottedNodes: Node[];
    private valueChanged;
    control: HTMLInputElement;
    protected proxy: HTMLInputElement;
    constructor();
    connectedCallback(): void;
    handleTextInput(): void;
}

export declare interface TextField extends StartEnd {
}

export declare enum TextFieldAppearance {
    filled = "filled",
    outline = "outline"
}

export declare const TextFieldTemplate: import("@microsoft/fast-element").ViewTemplate<TextField, any>;

export declare enum TextFieldType {
    email = "email",
    password = "password",
    tel = "tel",
    text = "text",
    url = "url"
}

/**
 * This file enables typing support for ElementInternals APIs.
 * It is largely taken from https://github.com/microsoft/TSJS-lib-generator/pull/818/files.
 *
 * When TypeScript adds support for these APIs we can delete this file.
 */
declare interface ValidityStateFlags {
    badInput?: boolean;
    customError?: boolean;
    patternMismatch?: boolean;
    rangeOverflow?: boolean;
    rangeUnderflow?: boolean;
    stepMismatch?: boolean;
    tooLong?: boolean;
    tooShort?: boolean;
    typeMismatch?: boolean;
    valueMissing?: boolean;
}

export { }
